#!/usr/bin/env python
# -*- coding: utf-8 -*-
import click
import os
import pathlib
from rastasteady.rastasteady import RastaSteady

@click.command(no_args_is_help=True, context_settings=dict(max_content_width=120))

@click.argument('video', type=str, required=True)

@click.option('--debug', type=bool, is_flag=True, default=False, help='Muestra informacion durante los procesos del video.', show_default=True)
@click.option('--dual', type=bool, is_flag=True, default=False, help='Crea fichero dual con video original y procesado. Requiere efecto RastaView.', show_default=True)
@click.option('--no-rastaview', type=bool, is_flag=True, default=False, help='No crea efecto RastaView del video. Require estabilizar video.', show_default=True)
@click.option('--no-reusar', type=bool, is_flag=True, default=False, help='Borra ficheros temporales si existen.', show_default=True)
@click.option('--perfil', type=click.Choice(['fast', 'default', 'slow'], case_sensitive=False), default='default', help='Perfil de estabilizado: rapido (peor estabilización), normal, lento (mejor estabilización).', show_default=True)
@click.option('--recortar', type=bool, is_flag=True, default=False, help='Recorta el fichero final para eliminar distorsion. Requiere efecto RastaView.', show_default=True)

@click.version_option('0.2.9')

def cli(video, debug, dual, no_rastaview, no_reusar, perfil, recortar):
    """RastaSteady es un software de estabilizacion de video para el sistema DJI FPV digital."""
    click.echo('Procesando %s!' % video)

    inputpathlib = pathlib.Path(video)
    currpathlib = pathlib.Path(str(inputpathlib.cwd()) + '/.placeholder')
    tmppathlib = pathlib.Path(str(inputpathlib.cwd()) + '/.rastasteady-' + inputpathlib.name + '/.placeholder')

    if no_reusar:
        for file in ['transforms.trf', 'xmap.gpm', 'ymap.gmp', 'cropped.mp4', 'dual.mp4', 'rastaview.mp4', 'stabilized.mp4']:
            if tmppathlib.with_name(file).is_file():
                tmppathlib.with_name(file).unlink()

    myVideo = RastaSteady(video, debug=debug)
    myVideo.stabilize(perfil)
    if not no_rastaview:
        myVideo.rastaview()
    if dual:
        myVideo.dual()
    if recortar:
        myVideo.crop()

    for file in ['cropped.mp4', 'dual.mp4', 'rastaview.mp4', 'stabilized.mp4']:
        if tmppathlib.with_name(file).is_file():
            os.rename(tmppathlib.with_name(file), currpathlib.with_name(inputpathlib.stem + '-' + file))



import sys
import math
import subprocess
import os
import pathlib
import time
from subprocess import check_output

class RastaSteady:
   def __init__(self, filename = '', debug = False):
      self.debug = debug
      self.pathlib = pathlib.Path(filename)
      self.tmppathlib = pathlib.Path(str(self.pathlib.cwd()) + '/.rastasteady-' + self.pathlib.name + '/.placeholder')

      self.inputfile = str(self.pathlib)

      try:
          self.tmppathlib.parent.mkdir()
      except FileExistsError:
          pass
      except:
          print('Error: error creating temporary directory')
          exit()

      self.getresolution()

   def tmpfile(self, filename):
      return str(self.tmppathlib.with_name(filename))

   def tmpfileexists(self, filename):
      return self.tmppathlib.with_name(filename).is_file()

   def debugopts(self):
      if self.debug == False: return '-hide_banner -loglevel warning -stats'
      return ''

   def derp_it(self, tx, target_width, src_width):
      x = (float(tx) / target_width - 0.5) * 2
      sx = tx - (target_width - src_width) / 2
      offset = math.pow(x, 2) * (-1 if x < 0 else 1) * ((target_width - src_width) / 2)
      return sx - offset

   def getresolution(self):
      out = subprocess.Popen(['ffprobe', '-v', 'error','-show_entries','stream=width,height','-of','csv=p=0:s=x', self.inputfile],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT) ## if STDERROR !!!
      stdout, stderr = out.communicate()
      cleanmsg = stdout.decode('utf-8')
      listamedida = cleanmsg.split('x')
      self.src_width = int(listamedida[0].strip('\n'))
      height = listamedida[1].strip('\n')
      self.height = int(height)

   def dual(self):
      if not self.tmpfileexists('dual.mp4'):
          if self.tmpfileexists('rastaview.mp4'):
              video = 'rastaview.mp4'
          elif self.tmpfileexists('stabilized.mp4'):
              video = 'stabilized.mp4'
          command = 'ffmpeg ' + self.debugopts() + ' -y -i ' + self.inputfile + ' -i ' + self.tmpfile(video) + ' -filter_complex hstack ' + self.tmpfile('dual.mp4')
          os.system(command)

   def stabilize(self, profile='default'):
      options = {
          'fast': ':zoom=0:optzoom=1:interpol=linear:smoothing=5',
          'default': ':zoom=0:optzoom=1:interpol=bilinear:smoothing=10,unsharp=5:5:0.8:3:3:0.4',
          'slow': ':zoom=1:optzoom=1:interpol=bicubic:smoothing=30,unsharp=5:5:0.8:3:3:0.4'
      }

      if not self.tmpfileexists('transforms.trf'):
          command1 = 'ffmpeg ' + self.debugopts() + ' -y -i ' + self.inputfile + ' -vf vidstabdetect=shakiness=10:accuracy=15:result=' + self.tmpfile('transforms.trf') + ' -f null -'
          os.system(command1)
      if not self.tmpfileexists('stabilized.mp4'):
          command2 = 'ffmpeg ' + self.debugopts() + ' -y -i ' + self.inputfile + ' -vf vidstabtransform=input="' + self.tmpfile('transforms.trf') + options[profile] + '" -acodec copy -vcodec libx264 -level 50 ' + self.tmpfile('stabilized.mp4')
          print(command2)
          os.system(command2)

   def crop(self):
      command = 'ffmpeg ' + self.debugopts() + ' -y -i ' + self.tmpfile('rastaview.mp4') + ' -vf "crop=in_w-80:in_h-60" '+ self.tmpfile('cropped.mp4')
      os.system(command)

   def rastaview(self, percentage = 20):
      if percentage == 0:
         return
      target_width = self.src_width + (self.src_width/100*int(percentage))
      target_width = int(target_width)

      xmap = open(self.tmpfile('xmap.pgm'), 'w')
      xmap.write('P2 {0} {1} 65535\n'.format(target_width, self.height))
      for y in range(self.height):
         for x in range(target_width):
            fudgeit = self.derp_it(x, target_width, self.src_width)
            xmap.write('{0} '.format(int(fudgeit)))
         xmap.write('\n')
      xmap.close()

      ymap = open(self.tmpfile('ymap.pgm'), 'w')
      ymap.write('P2 {0} {1} 65535\n'.format(target_width, self.height))
      for y in range(self.height):
         for x in range(target_width):
            ymap.write('{0} '.format(y))
         ymap.write('\n')
      ymap.close()

      if not self.tmpfileexists('rastaview.mp4'):
          command = 'ffmpeg ' + self.debugopts() + ' -y -i ' + self.tmpfile('stabilized.mp4') + ' -i ' + self.tmpfile('xmap.pgm') + ' -i ' + self.tmpfile('ymap.pgm') + ' -filter_complex remap,format=yuv420p -vcodec libx264 -level 50 ' + self.tmpfile('rastaview.mp4')
          os.system(command)
